@@ -1107,7 +1107,7 @@ xfce_panel_plugin_menu_remove (XfcePanelPlugin *plugin)
   gtk_window_set_screen (GTK_WINDOW (dialog),
       gtk_widget_get_screen (GTK_WIDGET (plugin)));
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-      _("If you remove the item from the panel, it is permanently lost."));
+      _("Removing the item from the panel also means its configuration will be lost."));
   gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Cancel"),
       GTK_RESPONSE_NO, _("_Remove"), GTK_RESPONSE_YES, NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_NO);
@@ -1197,26 +1197,6 @@ xfce_panel_plugin_menu_panel_help (XfcePanelPlugin *plugin)
 
 
 
-static void
-xfce_panel_plugin_menu_destroy (XfcePanelPlugin *plugin)
-{
-  GSList *li;
-
-  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
-
-  if (plugin->priv->menu != NULL)
-    {
-      /* remove custom items before they get destroyed */
-      for (li = plugin->priv->menu_items; li != NULL; li = li->next)
-        gtk_container_remove (GTK_CONTAINER (plugin->priv->menu), GTK_WIDGET (li->data));
-
-      gtk_menu_detach (GTK_MENU (plugin->priv->menu));
-      plugin->priv->menu = NULL;
-    }
-}
-
-
-
 static GtkMenu *
 xfce_panel_plugin_menu_get (XfcePanelPlugin *plugin)
 {
@@ -1241,6 +1221,10 @@ xfce_panel_plugin_menu_get (XfcePanelPlugin *plugin)
       gtk_widget_set_sensitive (item, FALSE);
       gtk_widget_show (item);
 
+      /* add custom menu items */
+      for (li = plugin->priv->menu_items; li != NULL; li = li->next)
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), GTK_WIDGET (li->data));
+
       /* separator */
       item = gtk_separator_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1293,10 +1277,6 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 G_GNUC_END_IGNORE_DEPRECATIONS
           gtk_widget_show (image);
 
-          /* add custom menu items */
-          for (li = plugin->priv->menu_items; li != NULL; li = li->next)
-            gtk_menu_shell_append (GTK_MENU_SHELL (menu), GTK_WIDGET (li->data));
-
           /* separator */
           item = gtk_separator_menu_item_new ();
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -2316,7 +2296,8 @@ xfce_panel_plugin_add_action_widget (XfcePanelPlugin *plugin,
  * @item   : a #GtkMenuItem.
  *
  * Insert a custom menu item to the plugin's right click menu. This item
- * is packed below the "Move" menu item.
+ * is packed below the first item in the menu, which displays the plugin's
+ * name.
  **/
 void
 xfce_panel_plugin_menu_insert_item (XfcePanelPlugin *plugin,
@@ -2407,6 +2388,43 @@ xfce_panel_plugin_menu_show_about (XfcePanelPlugin *plugin)
 
 
 
+/**
+ * xfce_panel_plugin_menu_destroy:
+ * @plugin : an #XfcePanelPlugin.
+ *
+ * Remove all custom menu items added through #xfce_panel_plugin_menu_insert_item
+ * from the menu.
+ **/
+ void
+ xfce_panel_plugin_menu_destroy (XfcePanelPlugin *plugin)
+ {
+   GSList *li;
+
+   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
+   panel_return_if_fail (XFCE_PANEL_PLUGIN_CONSTRUCTED (plugin));
+
+   if (plugin->priv->menu != NULL)
+     {
+       /* remove custom items before they get destroyed */
+       for (li = plugin->priv->menu_items; li != NULL; li = li->next)
+         {
+           if (GTK_IS_WIDGET (li->data))
+             {
+               gtk_container_remove (GTK_CONTAINER (plugin->priv->menu), GTK_WIDGET (li->data));
+               g_object_unref (li->data);
+             }
+           xfce_panel_plugin_menu_item_destroy (GTK_WIDGET (li->data), plugin);
+         }
+       g_slist_free (li);
+
+       gtk_menu_detach (GTK_MENU (plugin->priv->menu));
+       plugin->priv->menu = NULL;
+     }
+ }
+
+
+
+
 /**
  * xfce_panel_plugin_get_locked:
  * @plugin : an #XfcePanelPlugin.
@@ -2630,7 +2648,6 @@ xfce_panel_plugin_position_widget (XfcePanelPlugin *plugin,
   GdkRectangle    geometry;
   GdkDisplay     *display;
   GdkMonitor     *monitor;
-  GTimeVal        now_t, end_t;
   GtkWidget      *toplevel, *plug;
   gint            px, py;
   GtkAllocation   alloc;
@@ -2674,8 +2691,7 @@ xfce_panel_plugin_position_widget (XfcePanelPlugin *plugin,
    * use the coordinates */
   if (plugin->priv->panel_lock > 0)
     {
-      g_get_current_time (&end_t);
-      g_time_val_add (&end_t, G_USEC_PER_SEC / 2);
+      gint64 end_t = g_get_monotonic_time () + G_USEC_PER_SEC / 2;
 
       while (*x == -9999 && *y == -9999)
         {
@@ -2685,10 +2701,7 @@ xfce_panel_plugin_position_widget (XfcePanelPlugin *plugin,
           gdk_window_get_position (gtk_widget_get_window (attach_widget), x, y);
 
           /* don't try longer then 1/2 a second */
-          g_get_current_time (&now_t);
-          if (now_t.tv_sec > end_t.tv_sec
-              || (now_t.tv_sec == end_t.tv_sec
-                  && now_t.tv_usec > end_t.tv_usec))
+          if (g_get_monotonic_time () > end_t)
             break;
         }
     }
