@@ -101,6 +101,10 @@ struct _PagerPlugin
   XfcePanelPlugin __parent__;
 
   GtkWidget     *pager;
+  GObject       *numbering_switch;
+  GObject       *numbering_label;
+  GObject       *scrolling_switch;
+  GObject       *scrolling_label;
 
   WnckScreen    *wnck_screen;
 
@@ -109,6 +113,7 @@ struct _PagerPlugin
   guint          wrap_workspaces : 1;
   guint          miniature_view : 1;
   gint           rows;
+  gboolean       numbering;
   gfloat         ratio;
 };
 
@@ -118,7 +123,8 @@ enum
   PROP_WORKSPACE_SCROLLING,
   PROP_WRAP_WORKSPACES,
   PROP_MINIATURE_VIEW,
-  PROP_ROWS
+  PROP_ROWS,
+  PROP_NUMBERING
 };
 
 
@@ -181,6 +187,13 @@ pager_plugin_class_init (PagerPluginClass *klass)
                                                       NULL, NULL,
                                                       1, 50, 1,
                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_NUMBERING,
+                                   g_param_spec_boolean ("numbering",
+                                                         NULL, NULL,
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 
@@ -193,6 +206,7 @@ pager_plugin_init (PagerPlugin *plugin)
   plugin->wrap_workspaces = FALSE;
   plugin->miniature_view = TRUE;
   plugin->rows = 1;
+  plugin->numbering = FALSE;
   plugin->ratio = 1.0;
   plugin->pager = NULL;
 }
@@ -220,6 +234,15 @@ pager_plugin_get_property (GObject    *object,
     case PROP_MINIATURE_VIEW:
       g_value_set_boolean (value, plugin->miniature_view);
 
+      if (G_IS_OBJECT (plugin->numbering_label))
+        gtk_widget_set_visible (GTK_WIDGET (plugin->numbering_label), !plugin->miniature_view);
+      if (G_IS_OBJECT (plugin->numbering_switch))
+        gtk_widget_set_visible (GTK_WIDGET (plugin->numbering_switch), !plugin->miniature_view);
+      if (G_IS_OBJECT (plugin->scrolling_label))
+        gtk_widget_set_visible (GTK_WIDGET (plugin->scrolling_label), !plugin->miniature_view);
+      if (G_IS_OBJECT (plugin->scrolling_switch))
+        gtk_widget_set_visible (GTK_WIDGET (plugin->scrolling_switch), !plugin->miniature_view);
+
       pager_plugin_screen_layout_changed (plugin);
       break;
 
@@ -227,6 +250,10 @@ pager_plugin_get_property (GObject    *object,
       g_value_set_uint (value, plugin->rows);
       break;
 
+    case PROP_NUMBERING:
+      g_value_set_boolean (value, plugin->numbering);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -273,6 +300,14 @@ pager_plugin_set_property (GObject      *object,
         }
       break;
 
+    case PROP_NUMBERING:
+      plugin->numbering = g_value_get_boolean (value);
+
+      if (plugin->pager != NULL
+          && !plugin->miniature_view)
+        pager_buttons_set_numbering (XFCE_PAGER_BUTTONS (plugin->pager), plugin->numbering);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -451,6 +486,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       plugin->pager = pager_buttons_new (plugin->wnck_screen);
       pager_buttons_set_n_rows (XFCE_PAGER_BUTTONS (plugin->pager), plugin->rows);
       pager_buttons_set_orientation (XFCE_PAGER_BUTTONS (plugin->pager), orientation);
+      pager_buttons_set_numbering (XFCE_PAGER_BUTTONS (plugin->pager), plugin->numbering);
     }
 
   gtk_container_add (GTK_CONTAINER (plugin), plugin->pager);
@@ -502,6 +538,7 @@ pager_plugin_construct (XfcePanelPlugin *panel_plugin)
     { "wrap-workspaces", G_TYPE_BOOLEAN },
     { "miniature-view", G_TYPE_BOOLEAN },
     { "rows", G_TYPE_UINT },
+    { "numbering", G_TYPE_BOOLEAN },
     { NULL }
   };
 
@@ -678,14 +715,15 @@ pager_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   g_signal_connect (G_OBJECT (object), "clicked",
       G_CALLBACK (pager_plugin_configure_workspace_settings), dialog);
 
-  object = gtk_builder_get_object (builder, "workspace-scrolling");
-  panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (object));
+  plugin->scrolling_label = gtk_builder_get_object (builder, "workspace-scrolling-label");
+  plugin->scrolling_switch = gtk_builder_get_object (builder, "workspace-scrolling");
+  panel_return_if_fail (GTK_IS_SWITCH (plugin->scrolling_switch));
   g_object_bind_property (G_OBJECT (plugin), "workspace-scrolling",
-                          G_OBJECT (object), "active",
+                          G_OBJECT (plugin->scrolling_switch), "active",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
-  object = gtk_builder_get_object (builder, "miniature-view");
-  panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (object));
+  object = gtk_builder_get_object (builder, "appearance");
+  panel_return_if_fail (GTK_IS_COMBO_BOX (object));
   g_object_bind_property (G_OBJECT (plugin), "miniature-view",
                           G_OBJECT (object), "active",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
@@ -696,6 +734,18 @@ pager_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
                           G_OBJECT (object), "value",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
+  plugin->numbering_label = gtk_builder_get_object (builder, "numbering-label");
+  plugin->numbering_switch = gtk_builder_get_object (builder, "numbering");
+  panel_return_if_fail (GTK_IS_SWITCH (plugin->numbering_switch));
+  g_object_bind_property (G_OBJECT (plugin), "numbering",
+                          G_OBJECT (plugin->numbering_switch), "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
+
+  gtk_widget_set_visible (GTK_WIDGET (plugin->numbering_label), !plugin->miniature_view);
+  gtk_widget_set_visible (GTK_WIDGET (plugin->numbering_switch), !plugin->miniature_view);
+  gtk_widget_set_visible (GTK_WIDGET (plugin->scrolling_label), !plugin->miniature_view);
+  gtk_widget_set_visible (GTK_WIDGET (plugin->scrolling_switch), !plugin->miniature_view);
+
   /* update the rows limit */
   pager_plugin_configure_n_workspaces_changed (plugin->wnck_screen, NULL, builder);
 
